//
// parser.cpp
//

#include <string>
#include <iostream>
using namespace std;

#include "parser.h"
#include "dictionary.h"
#include "vocabulary.h"

// constructor

/**
 * Initialises the parser object, clearing member variables.
 */

Parser::Parser (  )
{
	inputline.erase ( inputline.begin (  ), inputline.end (  ) );
}

/**
 * resets the data values in the object
 */

void Parser::reset (  )
{
	inputline.erase ( inputline.begin (  ), inputline.end (  ) );
}

/**
 * this method accepts the raw input and outputs a formatted string, cleaning
 * it of unwanted characters
 * \return the new formatted string
 */

string Parser::clean_input (  )
{
	int i;

	string rawinput;
	string newstring;

	getline ( cin, rawinput );
	int size = rawinput.size (  );

	// loop through the string

	for ( i = 0; i < size; i++ )
	{
		// only accept alpha characters and convert them to upper case
		if ( isalpha ( rawinput[i] ) )
		{
			newstring += toupper ( rawinput[i] );
		}
		else if ( rawinput[i] == ' ' )
		{
			// we only allow a space if the previous character isn't a
			// space and not the first char in our new string
			if ( ( newstring[newstring.size (  ) - 1] != ' ' ) &&
					 ( newstring.size (  ) != 0 ) )
			{
				newstring += rawinput[i];
			}
		}
		else if ( i == 0 && rawinput[i] == '/' )
		{
			newstring += rawinput[i];
		}
	}

	// strip off the trailing space
	while ( newstring[newstring.size (  ) - 1] == ' ' )
	{
		newstring[newstring.size (  ) - 1] = '\0';
	}

	return newstring;

}

/**
 * gets input from cin and places it into the rawinput string
 */

void Parser::input (  )
{

	// get the formated input 
	inputline = clean_input (  );
}

/**
 * \return 1 if the input is empty
 * \return 0 if the input is not empty
 */

int Parser::empty (  )
{
	if ( inputline.size (  ) == 0 )
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

/**
 * \return the current value of the inputline
 */

string Parser::inputvalue (  )
{
	return inputline;
}

/**
 * Works out if the currently input is a command.
 * \retval 1 Command was inputed
 * \retval 0 No command inputed.
 */

int Parser::is_command (  )
{
	if ( inputline.at ( 0 ) == '/' )	// we will use the slash '/' for now.
	{
		return 1;
	}
	else
	{
		return 0;
	}
}

/**
 * Works out if the currently entered command is 'QUIT'
 * \retval 1 current command is 'QUIT'
 * \retval 0 current command is not 'QUIT'
 */

int Parser::cmd_quit (  )
{
	if ( inputline == "/QUIT" )
		return 1;
	else
		return 0;
}

/**
 * Works out if the currently entered command is 'HELP'
 * \retval 1 current command is 'HELP'
 * \retval 0 current command is not 'HELP'
 */

int Parser::cmd_help (  )
{
	if ( inputline == "/HELP" )
		return 1;
	else
		return 0;
}

/**
 * Works out if the currently entered command is 'STATUS'
 * \retval 1 current command is 'STATUS'
 * \retval 0 current command is not 'STATUS'
 */

int Parser::cmd_status (  )
{
	if ( inputline == "/STATUS" )
		return 1;
	else
		return 0;
}

/**
 * Works out if the currently entered command is 'RESET'
 * \retval 1 current command is 'RESET'
 * \retval 0 current command is not 'RESET'
 */

int Parser::cmd_reset (  )
{
	if ( inputline == "/RESET" )
		return 1;
	else
		return 0;
}

/**
 * Works out if the currently entered command is 'ERASE ME'
 * \retval 1 current command is 'ERASE ME'
 * \retval 0 current command is not 'ERASE ME'
 */

int Parser::cmd_erase (  )
{
	if ( inputline == "/ERASE" )
	{
		cout << "If you REALLY want to erase NICOLE, then use /ERASE ME" << endl;
		return 0;
	}

	if ( inputline == "/ERASE ME" )
		return 1;
	else
		return 0;
}

/**
 * Display the help screen to stdout
 */

void Parser::show_help (  )
{
	cout << "Commands:" << endl
		<< "---------" << endl
		<< "/HELP     - Shows this help screen" << endl
		<< "/STATUS   - Display the current status of NICOLE" << endl
		<< "/RESET    - Resets NICOLE's conversation path (warm boot)" << endl
		<< "/ERASE ME - Brainwashes NICOLE. **WARNING: Unrecoverable!**" << endl
		<< "/QUIT     - Quits NICOLE" << endl << endl;

}

/**
 * Display the status of NICOLE to stdout
 */

void Parser::show_status (  )
{
	Dictionary d;
	Vocabulary v;

	cout << "Generating status. Please wait..." << endl << endl;

	cout << "NICOLE Status:" << endl
		<< "--------------" << endl
		<< "Dictionary size: " << d.count (  ) << " words." << endl
		<< "Vocabulary size: " << v.count (  ) << " phrases." << endl << endl;

	cout << "done." << endl << endl;
}

/**
 * Erases the contents on NICOLE's brain. Not a very nice thing to do
 */

void Parser::erase_nicole (  )
{
	Dictionary d;
	Vocabulary v;

	cout << "Brainwashing NICOLE..." << endl;

	d.erase (  );

	cout << "AAAAAAAAAaaaaaaaaaaaaaaaaaaarrrrrrrrrrrgggggggggghhhhhhhhhh!!!!!"
		<< endl;

	v.erase (  );

	cout << "NICOLE is now brain dead." << endl;
}

/**
 * Destructs the parser object.
 * This doesn't do much as I don't dynamically create variables here.
 */

Parser::~Parser (  )
{
}
